import java.io.*;

/** 
 *  <p>The <tt>SourceFileTokenizer</tt> class opens an existing file and uses it as 
 *  the input stream.  It is then parsed into "tokens" that can be read one at a time. 
 *  The tokenizer recognizes Java source code identifiers.  It skips over numbers, 
 *  quoted strings, and various comment styles.</p>
 *
 *  <p>A typical application first constructs an instance of this class, and then repeatedly 
 *  loops calling the <tt>nextSourceToken</tt> method in each iteration of the loop until it returns 
 *  the value <tt>null</tt>.  The following is one example of the use of the tokenizer.</p>
 *
 *  <p>The code:</p>
 *
 *  <pre>
 *  SourceFileTokenizer sft = new SourceFileTokenizer( "<a href="HelloWorld.java">HelloWorld.java</a>" );
 *  String token = sft.nextSourceToken();
 *  while ( token != null ) {
 *      System.out.println( token );
 *      token = sft.nextSourceToken());
 *  }</pre>
 *
 *  <p>prints the following output:</p>
 *
 *  <pre>
 *  public
 *  class
 *  HelloWorld
 *  public
 *  static
 *  void
 *  main
 *  String
 *  args
 *  System
 *  out
 *  println</pre>
 *
 *  <p>View the <a href="SourceFileTokenizer.java">source code</a></p>
 *
 *  @author Deb Deppeler
 */
public class SourceFileTokenizer extends java.io.StreamTokenizer {

    /**
     *  Opens a text file for tokenizing.
     *  Only strings of characters (identifiers) are returned, all punctation is ignored.
	 *  All comments are also ignored.
     *
     *  @throws FileNotFoundException if the file does not exist
     */
    public SourceFileTokenizer ( String fileName ) throws FileNotFoundException {
        super( new BufferedReader( new FileReader( new File(fileName) ) ) );
        slashSlashComments(true);  // ignore // comments
        slashStarComments(true);   // ignore /* comments */
    }

    /** 
     *  Returns the next identifier in the source code file.
     *
     *  @return The next non-whitespace, non-comment string of characters read from a file. 
     *  The value <tt>null</tt> is returned if there are no remaining identifiers to read.
     * 
     *  @throws IOException if an I/O error occurs
     */
    public String nextSourceToken() throws IOException {

        // treat object.method as two separate identifiers
        ordinaryChars('.', '.');

        // treat TT_WORD as one identifier
        wordChars( '_', '_' );

        // ignore all punctuation and any token that is not a word
        while ( nextToken() != TT_EOF && ttype != TT_WORD ) ;  

        // return the word
        return sval;
    }

}

